import os
import unittest
from pathlib import Path
import glob
import shutil

from dotfile_manager.repo import Repo


class DotfilesRepoTest(unittest.TestCase):

    def setUp(self):
        repo_path = Path(__file__).parent / 'test_data/dotfiles_repo'
        self.repo = Repo(repo_path)
        self._clean()

    def tearDown(self):
        self._clean()

    def _clean(self):
        install_markers = glob.glob('topic_*_install_*.txt', recursive=False)
        for marker_file in install_markers:
            os.remove(marker_file)

        source_markers = glob.glob('topic_*_source.txt', recursive=False)
        for marker_file in source_markers:
            os.remove(marker_file)

        shutil.rmtree(self.repo.path / 'generated', ignore_errors=True)

    def test_install_all_macos(self):
        self.repo.install_all('macos')
        self.assertTrue(os.path.exists('topic_a_install_macos.txt'))
        self.assertTrue(os.path.exists('topic_b_install_macos.txt'))
        self.assertFalse(os.path.exists('topic_c_install_macos.txt'))
        self.assertFalse(os.path.exists('topic_c_install_ubuntu.txt'))

    def test_install_macos(self):
        self.repo.install('topic_a', 'macos')
        self.assertTrue(os.path.exists('topic_a_install_macos.txt'))
        self.assertFalse(os.path.exists('topic_b_install_macos.txt'))
        self.assertFalse(os.path.exists('topic_c_install_macos.txt'))
        self.assertFalse(os.path.exists('topic_c_install_ubuntu.txt'))

    def test_install_all_ubuntu(self):
        self.repo.install_all('ubuntu')
        self.assertTrue(os.path.exists('topic_a_install_ubuntu.txt'))
        self.assertTrue(os.path.exists('topic_b_install_ubuntu.txt'))
        self.assertFalse(os.path.exists('topic_c_install_macos.txt'))
        self.assertFalse(os.path.exists('topic_c_install_ubuntu.txt'))

    def test_install_ubuntu(self):
        self.repo.install('topic_a', 'ubuntu')
        self.assertTrue(os.path.exists('topic_a_install_ubuntu.txt'))
        self.assertFalse(os.path.exists('topic_b_install_ubuntu.txt'))
        self.assertFalse(os.path.exists('topic_c_install_macos.txt'))
        self.assertFalse(os.path.exists('topic_c_install_ubuntu.txt'))

    def test_setup(self):
        self.repo.setup_all()

        # Test that generated symlinks to binaries are available:
        self.assertTrue(os.path.exists(self.repo.get_path() / 'generated/bin/executable_from_a.sh'))
        self.assertTrue(os.path.exists(self.repo.get_path() / 'generated/bin/executable_from_b.sh'))
        self.assertTrue(os.path.exists(
            self.repo.get_path() / 'generated/bin/another_executable_from_b.sh'))
        self.assertFalse(os.path.exists(
            self.repo.get_path() / 'generated/bin/executable_from_c.sh'))
        self.assertFalse(os.path.exists(
            self.repo.get_path() / 'generated/bin/another_executable_from_c.sh'))

        # Test that generated symlinks to general files are available:
        home = Path.home()
        self.assertTrue(os.path.exists(home / 'symlink_replica_from_a.txt'))
        self.assertTrue(os.path.exists(home / 'symlink_replica_from_b.txt'))
        self.assertFalse(os.path.exists(home / 'symlink_replica_from_c.txt'))

        # Test that proper files are sourced:
        os.system(f'sh {self.repo.get_path()/"generated/sources.sh"}')
        self.assertTrue(os.path.exists('topic_a_source.txt'))
        self.assertTrue(os.path.exists('topic_b_source.txt'))
        self.assertFalse(os.path.exists('topic_c_source.txt'))


if __name__ == '__main__':
    unittest.main()



import os
from pathlib import Path
import subprocess

import yaml

from dotfile_manager.config import PROJECT_CONFIG_NAME


class Project:

    def __init__(self, path: Path) -> None:
        self.path = path
        self.name = self.path.name

    def get_name(self) -> str:
        return self.name

    def is_valid_project(self) -> bool:
        return self.path.is_dir() and (self.path / PROJECT_CONFIG_NAME).exists()

    def is_disabled(self) -> bool:
        with open(self.path / PROJECT_CONFIG_NAME) as file:
            try:
                # This key is optional. If not specified we assume the project to be enabled.
                is_disabled = yaml.load(file, Loader=yaml.FullLoader)['disable']
                return is_disabled
            except KeyError:
                return False

    def install(self, os) -> None:
        with open(self.path / PROJECT_CONFIG_NAME) as file:
            try:
                install_scripts = yaml.load(file, Loader=yaml.FullLoader)[f'install_{os}']
            except KeyError:
                return

            for install_script in install_scripts:
                script = self.path / install_script
                assert script.exists(), script
                subprocess.run(script)

        print(f'Successfully installed project {self.name} for {os}.')

    def create_symbolic_links(self) -> None:
        with open(self.path / PROJECT_CONFIG_NAME) as file:
            try:
                symlinks = yaml.load(file, Loader=yaml.FullLoader)['symlinks']
            except KeyError:
                print('No configured symlinks found.')
                return

            for source, destination in symlinks.items():
                source_path = self.path / source
                assert source_path.exists(), source_path

                destination_path = Path(destination).expanduser()
                destination_path.parent.mkdir(parents=True, exist_ok=True)
                destination_path.unlink(missing_ok=True)

                os.symlink(source_path, destination_path)
                print(f'Created symlink from {source_path} to {destination_path}.')

    def create_bin(self, destination_folder: Path) -> None:
        with open(self.path / PROJECT_CONFIG_NAME) as file:
            try:
                binaries = yaml.load(file, Loader=yaml.FullLoader)['bin']
            except KeyError:
                print('No configured binaries found.')
                return

            for binary in binaries:
                binary_path = self.path / binary
                assert binary_path.exists(), binary_path

                # Use only binary_path.name s.t. we ignore the path if it is in a subfolder.
                destination = destination_folder / binary_path.name
                os.symlink(binary_path, destination)
                print(f'Created symlink from {binary_path} to {destination}.')

    def add_sources(self, output_file) -> None:
        with open(self.path / PROJECT_CONFIG_NAME) as file:
            try:
                source_files = yaml.load(file, Loader=yaml.FullLoader)['source']
            except KeyError:
                print('No configured sourcing files found.')
                return

            for source_file in source_files:
                source_path = self.path / source_file
                assert source_path.exists(), source_path

                output_file.write(f'. {source_path}\n')
                print(f'Added {source_path} to sourced in {source_path}.')



import os
import unittest
from pathlib import Path

from dotfile_manager.cli import create_parser, get_default_dotfiles_path


class CliTest(unittest.TestCase):

    def setUp(self):
        self.parser = create_parser()

    def test_dotfiles_path(self):
        args = self.parser.parse_args(['-d', '/tmp/foo', 'install'])
        self.assertEqual(args.verb, 'install')
        self.assertEqual(args.dotfiles, Path('/tmp/foo'))

        args = self.parser.parse_args(['install'])
        self.assertEqual(args.verb, 'install')
        self.assertEqual(args.dotfiles, get_default_dotfiles_path())

    def test_install(self):
        args = self.parser.parse_args(['install'])
        self.assertEqual(args.verb, 'install')
        self.assertEqual(args.verb, 'install')
        self.assertEqual(args.project, None)

        args = self.parser.parse_args(['install', 'topic_a'])
        self.assertEqual(args.verb, 'install')
        self.assertEqual(args.project, 'topic_a')

    def test_setup(self):
        args = self.parser.parse_args(['setup'])
        self.assertEqual(args.verb, 'setup')


if __name__ == '__main__':
    unittest.main()



from pathlib import Path
import argparse
import os

from dotfile_manager.cli import create_parser
from dotfile_manager.repo import Repo

def main():
    parser = create_parser()
    args = parser.parse_args()

    repo = Repo(args.dotfiles)
    if args.verb == 'install':
        if args.project:
            repo.install(args.project)
        else:
            repo.install_all()
        return

    if args.verb == 'setup':
        repo.setup_all()
        return

    print('Unknown verb, doing nothing.')
    return



from pathlib import Path
import argparse
import os

from dotfile_manager.repo import Repo
from dotfile_manager.version import __version__
from dotfile_manager.config import DOTFILES_PATH_ENV_VARIABLE


def get_default_dotfiles_path() -> Path:
    """
    Get the default dotfiles path. This is the env variable if set, else the current working
    directory.
    """
    if DOTFILES_PATH_ENV_VARIABLE in os.environ:
        return Path(os.environ[DOTFILES_PATH_ENV_VARIABLE])

    return Path(os.getcwd())


def create_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(description='Tool for managing dotfiles for linux and macos.')
    parser.add_argument('-d',
                        '--dotfiles',
                        type=Path,
                        default=get_default_dotfiles_path(),
                        help='Path to the dotfiles repository')
    parser.add_argument('-v',
                        '--version',
                        action='version',
                        version=__version__,
                        help='Print the version')

    verb_parser = parser.add_subparsers(dest='verb')
    verb_parser.required = True

    install_parser = verb_parser.add_parser('install', help='Install dependencies of dotfiles')
    install_parser.add_argument('project', type=str, nargs='?', help='Name of dotfile project that will be installed')

    _ = verb_parser.add_parser('setup', help='Setup dotfiles')
    return parser



from pathlib import Path

import setuptools

from src.dotfile_manager.version import __version__

file_path = Path(__file__)
with open(file_path.parent / 'README.md', 'r') as fh:
    LONG_DESCRIPTION = fh.read()

setuptools.setup(
    name='dotfile_manager',
    version=__version__,
    author='Lionel Gulich',
    author_email='lgulich@ethz.ch',
    url='https://github.com/lgulich/dotfile_manager',
    description='A tool for managing dotfiles for linux and macos.',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    install_requires=['pyyaml'],
    classifiers=[
        'Programming Language :: Python :: 3 :: Only',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: POSIX :: Linux',
        'Natural Language :: English',
    ],
    license='Closed',
    package_dir={'': 'src'},
    packages=setuptools.find_packages('src'),
    include_package_data=True,
    package_data={'dotfile_manager': ['py.typed']},
    scripts=[
        'scripts/dotfile_manager',
    ],
    python_requires='>=3.8',
)



from datetime import datetime
from pathlib import Path
import platform
import shutil
import os

from dotfile_manager.config import BIN_PATH, SOURCE_FILE_PATH
from dotfile_manager.project import Project


def get_os_name() -> str:
    platform_name = platform.system()
    # TODO(lgulich): Find way to also determine linux distro.
    if platform_name == 'Linux':
        return 'ubuntu'
    if platform_name == 'Darwin':
        return 'macos'
    raise


class Repo:

    def __init__(self, path: Path) -> None:
        self.path: Path = path

    def get_path(self) -> Path:
        return self.path

    def install_all(self, os_name: str = get_os_name()) -> None:
        for project_path in sorted(self.path.iterdir()):
            self.install(os.path.basename(project_path), os_name)
        print('Successfully installed all projects.')

    def install(self, project_name: str, os_name: str = get_os_name()) -> None:
            project = Project(self.path / project_name)
            if not project.is_valid_project():
                return
            if project.is_disabled():
                print(f'Project {project.get_name()} is disabled - Skipping.')
                return
            project.install(os_name)

    def setup_all(self) -> None:
        # Create folder into which all binaries will be symlinked.
        bin_path = self.path / BIN_PATH
        shutil.rmtree(bin_path, ignore_errors=True)
        bin_path.mkdir(parents=True)

        # Create file which contains everything that should be sourced.
        source_file_path = self.path / SOURCE_FILE_PATH
        source_file_path.unlink(missing_ok=True)
        source_file_path.parent.mkdir(parents=True, exist_ok=True)
        with open(source_file_path, 'a') as output_file:
            output_file.write(f'# Autogenerated on {datetime.now()}.\n')
            output_file.write('# shellcheck shell=sh\n\n')

            # Setup every project individually:
            for project_path in sorted(self.path.iterdir()):
                project = Project(project_path)
                if not project.is_valid_project():
                    continue
                if project.is_disabled():
                    print(f'Project {project.get_name()} is disabled - Skipping.')
                    continue
                print(f'Setting up {project.get_name()}:')
                project.create_symbolic_links()
                project.create_bin(bin_path)
                project.add_sources(output_file)

        print(f'Successfully setup all projects. Sourcing script created at "{source_file_path}". '
              f'Binaries located at "{bin_path}".')
